/***
 * Juliac
 * Copyright (C) 2008-2013 Inria, Univ. Lille 1
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Lionel Seinturier
 * Contributors: Valerio Schiavoni, Philippe Merle
 */

package org.objectweb.fractal.juliac.compile.jdt;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.jdt.internal.compiler.ClassFile;
import org.eclipse.jdt.internal.compiler.Compiler;
import org.eclipse.jdt.internal.compiler.IErrorHandlingPolicy;
import org.eclipse.jdt.internal.compiler.ast.CompilationUnitDeclaration;
import org.eclipse.jdt.internal.compiler.env.ICompilationUnit;
import org.eclipse.jdt.internal.compiler.env.INameEnvironment;
import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
import org.eclipse.jdt.internal.compiler.problem.DefaultProblemFactory;
import org.objectweb.fractal.juliac.Juliac;
import org.objectweb.fractal.juliac.SourceFile;

/**
 * This class extends the JDT compiler and stores all compiled types ({@link
 * CompilationUnitDeclaration}s.)
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @since 2.2.1
 */
class JDTCompiler extends Compiler {

    private List<CompilationUnitDeclaration> unitds;
    
    JDTCompiler(
        Juliac jc, INameEnvironment environment, IErrorHandlingPolicy policy,
        CompilerOptions options ) {
        
        super(
            environment, policy, options,
            new CompilerRequestorImpl(jc), new DefaultProblemFactory() );
    }
    
    @Override
    public void process( CompilationUnitDeclaration unit, int i ) {
        unitds.add(unit);
        super.process(unit,i);
    }
    
    void compile( Map<ICompilationUnit,SourceFile> units ) {
        
        unitds = new ArrayList<CompilationUnitDeclaration>();

        /*
         * Initialize the callback object which receives compilation results.
         */
        CompilerRequestorImpl r = (CompilerRequestorImpl) requestor;
        r.init(units);
        
        /*
         * Compile.
         */
        Set<ICompilationUnit> luts = units.keySet();
        ICompilationUnit[] uts =
            luts.toArray(new ICompilationUnit[luts.size()]);
        compile(uts);
    }
    
    List<CompilationUnitDeclaration> getUnits() {
        return unitds;
    }
    
    /**
     * @since 2.2.4
     */
    boolean hasError() {
        CompilerRequestorImpl r = (CompilerRequestorImpl) requestor;
        boolean hasError = r.hasErrors();
        return hasError;
    }
    
    List<ClassFile> getClassFiles() {
        CompilerRequestorImpl r = (CompilerRequestorImpl) requestor;
        List<ClassFile> classFiles = r.getClassFiles();
        return classFiles;
    }

    /**
     * Get the compilation error message.
     *
     * @return the compilation error message.
     *
     * @since 2.6
     */
    String getErrorMessage() {
        CompilerRequestorImpl r = (CompilerRequestorImpl) requestor;
        String errorMessage = r.getErrorMessage();
        return errorMessage;
    }
}
